@@ -1,3 +1,4 @@
+import asyncio
 import logging
 from typing import Dict, List, Optional, Tuple
 
@@ -37,9 +38,12 @@ class BlockStore:
             "block blob, sub_epoch_summary blob, is_peak tinyint, is_block tinyint)"
         )
 
+        # todo remove in v1.2
+        await self.db.execute("DROP TABLE IF EXISTS sub_epoch_segments")
+
         # Sub epoch segments for weight proofs
         await self.db.execute(
-            "CREATE TABLE IF NOT EXISTS sub_epoch_segments(ses_height bigint PRIMARY KEY, challenge_segments blob)"
+            "CREATE TABLE IF NOT EXISTS sub_epoch_segments_v2(ses_height bigint PRIMARY KEY, challenge_segments blob)"
         )
 
         # Height index so we can look up in order of height for sync purposes
@@ -106,7 +110,7 @@ class BlockStore:
         self, sub_epoch_summary_height: uint32, segments: List[SubEpochChallengeSegment]
     ):
         cursor_1 = await self.db.execute(
-            "INSERT OR REPLACE INTO sub_epoch_segments VALUES(?, ?)",
+            "INSERT OR REPLACE INTO sub_epoch_segments_v2 VALUES(?, ?)",
             (sub_epoch_summary_height, bytes(SubEpochSegments(segments))),
         )
         await cursor_1.close()
@@ -116,7 +120,7 @@ class BlockStore:
         sub_epoch_summary_height: uint32,
     ) -> Optional[List[SubEpochChallengeSegment]]:
         cursor = await self.db.execute(
-            "SELECT challenge_segments from sub_epoch_segments WHERE ses_height=?", (sub_epoch_summary_height,)
+            "SELECT challenge_segments from sub_epoch_segments_v2 WHERE ses_height=?", (sub_epoch_summary_height,)
         )
         row = await cursor.fetchone()
         await cursor.close()
@@ -125,7 +129,7 @@ class BlockStore:
         return None
 
     async def delete_sub_epoch_challenge_segments(self, fork_height: uint32):
-        cursor = await self.db.execute("delete from sub_epoch_segments WHERE ses_height>?", (fork_height,))
+        cursor = await self.db.execute("delete from sub_epoch_segments_v2 WHERE ses_height>?", (fork_height,))
         await cursor.close()
 
     async def get_full_block(self, header_hash: bytes32) -> Optional[FullBlock]:
@@ -188,6 +192,8 @@ class BlockStore:
         await cursor.close()
         ret: Dict[bytes32, HeaderBlock] = {}
         for row in rows:
+            # Ugly hack, until full_block.get_block_header is rewritten as part of generator runner change
+            await asyncio.sleep(0.001)
             header_hash = bytes.fromhex(row[0])
             full_block: FullBlock = FullBlock.from_bytes(row[1])
             ret[header_hash] = full_block.get_block_header()
