@@ -144,7 +144,9 @@ class FullNode:
 
     async def initialize_weight_proof(self):
         self.weight_proof_handler = WeightProofHandler(self.constants, self.blockchain)
-        await self.weight_proof_handler.get_proof_of_weight(self.blockchain.get_peak().header_hash)
+        peak = self.blockchain.get_peak()
+        if peak is not None:
+            await self.weight_proof_handler.get_proof_of_weight(self.blockchain.get_peak().header_hash)
 
     def set_server(self, server: ChiaServer):
         self.server = server
@@ -404,7 +406,7 @@ class FullNode:
             if peer is None:
                 await self.server.send_to_all([msg], NodeType.TIMELORD)
             else:
-                await peer.new_peak_timelord(timelord_new_peak)
+                await self.server.send_to_specific([msg], peer.peer_node_id)
 
     async def synced(self) -> bool:
         curr: Optional[BlockRecord] = self.blockchain.get_peak()
